@@ -25,49 +25,61 @@ import org.apache.velocity.app.Velocity;
 import org.apache.velocity.exception.MethodInvocationException;
 import org.apache.velocity.exception.ParseErrorException;
 import org.apache.velocity.exception.ResourceNotFoundException;
-import org.apache.velocity.runtime.log.LogSystem;
 import org.w3c.dom.Document;
+import org.w3c.dom.Element;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
 
 import com.ozacc.mail.Mail;
 import com.ozacc.mail.MailBuildException;
-import com.ozacc.mail.VelocityMailBuilder;
+import com.ozacc.mail.VelocityMultipleMailBuilder;
 
 /**
  * XMLեɤ߹ߡVelocityϢȤưŪ˥᡼ǡΥǡMail󥹥󥹤륯饹
  * 
  * @since 1.0.1
  * @author Tomohiro Otsuka
- * @version $Id: XMLVelocityMailBuilderImpl.java,v 1.6 2004/10/22 00:50:18 otsuka Exp $
+ * @version $Id: XMLVelocityMailBuilderImpl.java,v 1.11 2006/05/06 05:50:57 otsuka Exp $
  */
-public class XMLVelocityMailBuilderImpl extends XMLMailBuilderImpl implements VelocityMailBuilder {
+public class XMLVelocityMailBuilderImpl extends XMLMailBuilderImpl implements
+		VelocityMultipleMailBuilder {
 
 	private static Log log = LogFactory.getLog(XMLVelocityMailBuilderImpl.class);
 
-	private String charset = "UTF-8";
+	private static String CACHE_KEY_SEPARATOR = "#";
 
-	private LogSystem velocityLogSystem = new VelocityLogSystem();
+	private static String DEFAULT_MAIL_ID = "DEFAULT";
 
-	private Map templateCache = new HashMap();
+	protected String charset = "UTF-8";
+
+	static {
+		Velocity.setProperty(Velocity.RUNTIME_LOG_LOGSYSTEM, new VelocityLogSystem());
+		try {
+			Velocity.init();
+		} catch (Exception e) {
+			throw new MailBuildException("Velocityν˼Ԥޤ", e);
+		}
+	}
+
+	protected Map templateCache = new HashMap();
 
 	private boolean cacheEnabled = false;
 
-	private boolean hasTemplateCache(String key) {
+	protected boolean hasTemplateCache(String key) {
 		if (cacheEnabled) {
 			return templateCache.containsKey(key);
 		}
 		return false;
 	}
 
-	private void putTemplateCache(String key, String templateXmlText) {
+	protected void putTemplateCache(String key, String templateXmlText) {
 		if (cacheEnabled) {
 			log.debug("ƥץ졼Ȥ򥭥å夷ޤ[key='" + key + "']");
 			templateCache.put(key, templateXmlText);
 		}
 	}
 
-	private String getTemplateCache(String key) {
+	protected String getTemplateCache(String key) {
 		if (hasTemplateCache(key)) {
 			log.debug("ƥץ졼ȥå֤ޤ[key='" + key + "']");
 			return (String)templateCache.get(key);
@@ -104,8 +116,10 @@ public class XMLVelocityMailBuilderImpl extends XMLMailBuilderImpl implements Ve
 	 * @see com.ozacc.mail.VelocityMailBuilder#buildMail(java.lang.String, org.apache.velocity.VelocityContext)
 	 */
 	public Mail buildMail(String classPath, VelocityContext context) throws MailBuildException {
+		String cacheKey = classPath + CACHE_KEY_SEPARATOR + DEFAULT_MAIL_ID;
+
 		String templateXmlText;
-		if (!hasTemplateCache(classPath)) {
+		if (!hasTemplateCache(cacheKey)) {
 			Document doc;
 			try {
 				// VelocityޡXMLǤϥȤĤ
@@ -115,10 +129,10 @@ public class XMLVelocityMailBuilderImpl extends XMLMailBuilderImpl implements Ve
 			} catch (IOException e) {
 				throw new MailBuildException("XMLեɤ߹ߤ˼Ԥޤ", e);
 			}
-			templateXmlText = convertDocumentIntoString(doc);
-			putTemplateCache(classPath, templateXmlText);
+			templateXmlText = convertDocumentIntoString(doc.getDocumentElement());
+			putTemplateCache(cacheKey, templateXmlText);
 		} else {
-			templateXmlText = getTemplateCache(classPath);
+			templateXmlText = getTemplateCache(cacheKey);
 		}
 
 		try {
@@ -132,8 +146,10 @@ public class XMLVelocityMailBuilderImpl extends XMLMailBuilderImpl implements Ve
 	 * @see com.ozacc.mail.VelocityMailBuilder#buildMail(java.io.File, org.apache.velocity.VelocityContext)
 	 */
 	public Mail buildMail(File file, VelocityContext context) throws MailBuildException {
+		String cacheKey = file.getAbsolutePath() + CACHE_KEY_SEPARATOR + DEFAULT_MAIL_ID;
+
 		String templateXmlText;
-		if (!hasTemplateCache(file.getAbsolutePath())) {
+		if (!hasTemplateCache(cacheKey)) {
 			Document doc;
 			try {
 				// VelocityޡXMLǤϥȤĤ
@@ -143,10 +159,10 @@ public class XMLVelocityMailBuilderImpl extends XMLMailBuilderImpl implements Ve
 			} catch (IOException e) {
 				throw new MailBuildException("XMLեɤ߹ߤ˼Ԥޤ", e);
 			}
-			templateXmlText = convertDocumentIntoString(doc);
-			putTemplateCache(file.getAbsolutePath(), templateXmlText);
+			templateXmlText = convertDocumentIntoString(doc.getDocumentElement());
+			putTemplateCache(cacheKey, templateXmlText);
 		} else {
-			templateXmlText = getTemplateCache(file.getAbsolutePath());
+			templateXmlText = getTemplateCache(cacheKey);
 		}
 
 		try {
@@ -169,19 +185,17 @@ public class XMLVelocityMailBuilderImpl extends XMLMailBuilderImpl implements Ve
 	 * @throws ResourceNotFoundException
 	 * @throws IOException
 	 */
-	private Mail build(String templateXmlText, VelocityContext context)
-																		throws TransformerFactoryConfigurationError,
-																		Exception,
-																		ParseErrorException,
-																		MethodInvocationException,
-																		ResourceNotFoundException,
-																		IOException {
+	protected Mail build(String templateXmlText, VelocityContext context)
+																			throws TransformerFactoryConfigurationError,
+																			Exception,
+																			ParseErrorException,
+																			MethodInvocationException,
+																			ResourceNotFoundException,
+																			IOException {
 		if (log.isDebugEnabled()) {
 			log.debug("Source XML Mail Data\n" + templateXmlText);
 		}
 
-		Velocity.setProperty(Velocity.RUNTIME_LOG_LOGSYSTEM, velocityLogSystem);
-		Velocity.init();
 		StringWriter w = new StringWriter();
 		Velocity.evaluate(context, w, "XML Mail Data", templateXmlText);
 		StringReader reader = new StringReader(w.toString());
@@ -191,22 +205,22 @@ public class XMLVelocityMailBuilderImpl extends XMLMailBuilderImpl implements Ve
 		Document newDoc = db.parse(source);
 
 		if (log.isDebugEnabled()) {
-			String newXmlContent = convertDocumentIntoString(newDoc);
+			String newXmlContent = convertDocumentIntoString(newDoc.getDocumentElement());
 			log.debug("VelocityContext-merged XML Mail Data\n" + newXmlContent);
 		}
 
-		return buildMail(newDoc);
+		return buildMail(newDoc.getDocumentElement());
 	}
 
 	/**
 	 * ꤵ줿DOM DocumentʸѴޤ
 	 * 
-	 * @param doc
+	 * @param mailElement
 	 * @return XMLɥȤʸ
 	 * @throws TransformerFactoryConfigurationError 
 	 */
-	private String convertDocumentIntoString(Document doc)
-															throws TransformerFactoryConfigurationError {
+	protected String convertDocumentIntoString(Element mailElement)
+																	throws TransformerFactoryConfigurationError {
 		TransformerFactory tf = TransformerFactory.newInstance();
 		Transformer t;
 		try {
@@ -216,7 +230,7 @@ public class XMLVelocityMailBuilderImpl extends XMLMailBuilderImpl implements Ve
 		}
 		t.setOutputProperties(getOutputProperties());
 
-		DOMSource source = new DOMSource(doc);
+		DOMSource source = new DOMSource(mailElement);
 		StringWriter w = new StringWriter();
 		StreamResult result = new StreamResult(w);
 		try {
@@ -232,7 +246,7 @@ public class XMLVelocityMailBuilderImpl extends XMLMailBuilderImpl implements Ve
 	 * ϥץѥƥ
 	 * @return ϥץѥƥꤷProperties󥹥
 	 */
-	private Properties getOutputProperties() {
+	protected Properties getOutputProperties() {
 		Properties p = new Properties();
 		p.put(OutputKeys.ENCODING, charset);
 		p.put(OutputKeys.DOCTYPE_PUBLIC, Mail.DOCTYPE_PUBLIC);
@@ -240,4 +254,91 @@ public class XMLVelocityMailBuilderImpl extends XMLMailBuilderImpl implements Ve
 		return p;
 	}
 
+	/**
+	 * @see com.ozacc.mail.VelocityMultipleMailBuilder#buildMail(java.lang.String, org.apache.velocity.VelocityContext, java.lang.String)
+	 */
+	public Mail buildMail(String classPath, VelocityContext context, String mailId)
+																					throws MailBuildException {
+		if (mailId == null || "".equals(mailId)) {
+			throw new IllegalArgumentException("᡼IDꤵƤޤ");
+		}
+
+		String cacheKey = classPath + CACHE_KEY_SEPARATOR + mailId;
+
+		String templateXmlText;
+		if (!hasTemplateCache(cacheKey)) {
+			Document doc;
+			try {
+				// VelocityޡXMLǤϥȤĤ
+				doc = getDocumentFromClassPath(classPath, false);
+			} catch (SAXException e) {
+				throw new MailBuildException("XMLΥѡ˼Ԥޤ" + e.getMessage(), e);
+			} catch (IOException e) {
+				throw new MailBuildException("XMLեɤ߹ߤ˼Ԥޤ", e);
+			}
+			if (Mail.DOCTYPE_PUBLIC.equals(doc.getDoctype().getPublicId())) {
+				throw new MailBuildException("ꤵ줿饹ѥXMLϥ󥰥᡼ƥץ졼ȤǤ[classPath='"
+						+ classPath + "']");
+			}
+			templateXmlText = getAndCacheTemplateText(doc, mailId, cacheKey);
+		} else {
+			templateXmlText = getTemplateCache(cacheKey);
+		}
+
+		try {
+			return build(templateXmlText, context);
+		} catch (Exception e) {
+			throw new MailBuildException("᡼˼Ԥޤ", e);
+		}
+	}
+
+	private String getAndCacheTemplateText(Document doc, String mailId, String cacheKey)
+																						throws TransformerFactoryConfigurationError {
+		Element mailElem = doc.getElementById(mailId);
+		if (mailElem == null) {
+			throw new MailBuildException("ꤵ줿ID[" + mailId + "]Υ᡼ǡϸĤޤǤ");
+		}
+		String templateXmlText = templateXmlText = convertDocumentIntoString(mailElem);
+		putTemplateCache(cacheKey, templateXmlText);
+		return templateXmlText;
+	}
+
+	/**
+	 * @see com.ozacc.mail.VelocityMultipleMailBuilder#buildMail(java.io.File, org.apache.velocity.VelocityContext, java.lang.String)
+	 */
+	public Mail buildMail(File file, VelocityContext context, String mailId)
+																			throws MailBuildException {
+		if (mailId == null || "".equals(mailId)) {
+			throw new IllegalArgumentException("᡼IDꤵƤޤ");
+		}
+
+		String cacheKey = file.getAbsolutePath() + CACHE_KEY_SEPARATOR + mailId;
+
+		String templateXmlText;
+		if (!hasTemplateCache(cacheKey)) {
+			Document doc;
+			try {
+				// VelocityޡXMLǤϥȤĤ
+				doc = getDocumentFromFile(file, false);
+			} catch (SAXException e) {
+				throw new MailBuildException("XMLΥѡ˼Ԥޤ" + e.getMessage(), e);
+			} catch (IOException e) {
+				throw new MailBuildException("XMLեɤ߹ߤ˼Ԥޤ", e);
+			}
+			if (Mail.DOCTYPE_PUBLIC.equals(doc.getDoctype().getPublicId())) {
+				throw new MailBuildException("ꤵ줿եXMLϥ󥰥᡼ƥץ졼ȤǤ[filePath='"
+						+ file.getAbsolutePath() + "']");
+			}
+			templateXmlText = getAndCacheTemplateText(doc, mailId, cacheKey);
+		} else {
+			templateXmlText = getTemplateCache(cacheKey);
+		}
+
+		try {
+			return build(templateXmlText, context);
+		} catch (Exception e) {
+			throw new MailBuildException("᡼˼Ԥޤ", e);
+		}
+	}
+
 }
\ No newline at end of file
