@@ -11,17 +11,16 @@ import org.apache.velocity.VelocityContext;
 
 import com.ozacc.mail.Mail;
 import com.ozacc.mail.MailBuildException;
-import com.ozacc.mail.VelocityMailBuilder;
 
 /**
  * XMLMailBuilderΥƥȥ
  * 
  * @author Tomohiro Otsuka
- * @version $Id: JDomXMLMailBuilderTest.java,v 1.3 2004/09/21 11:55:19 otsuka Exp $
+ * @version $Id: JDomXMLMailBuilderTest.java,v 1.3.2.2 2005/02/01 20:38:14 otsuka Exp $
  */
 public class JDomXMLMailBuilderTest extends TestCase {
 
-	private VelocityMailBuilder builder;
+	private JDomXMLMailBuilder builder;
 
 	protected void setUp() throws Exception {
 		super.setUp();
@@ -190,15 +189,18 @@ public class JDomXMLMailBuilderTest extends TestCase {
 		String name = "";
 		String email = "misaki@example.com";
 		Customer customer = new Customer(name, email);
+		String item = "GIVE&TAKE (Beige)\n\nDesigned byRonan & Erwan Boroullec, SizeW313*D84*H370, Color꡼, ΡABS";
 
 		InternetAddress from = new InternetAddress("shop@example.com", "XMLMailBuilder饤󥷥å");
 		InternetAddress to = new InternetAddress(email, name);
 
 		String subject = "XMLMailBuilder饤󥷥å - ʸγǧ";
-		String text = name + "\n\n㤤夲꤬Ȥޤ";
+		String text = name
+				+ "\n\n㤤夲꤬Ȥޤ\n\nGIVE&TAKE (Beige)\n\nDesigned byRonan & Erwan Boroullec, SizeW313*D84*H370, Color꡼, ΡABS";
 
 		VelocityContext context = new VelocityContext();
 		context.put("customer", customer);
+		context.put("item", item);
 
 		// ᡼¹
 		Mail result = builder.buildMail(classPath, context);
@@ -219,15 +221,18 @@ public class JDomXMLMailBuilderTest extends TestCase {
 		String name = "";
 		String email = "misaki@example.com";
 		Customer customer = new Customer(name, email);
+		String item = "GIVE&TAKE (Beige)\n\nDesigned byRonan & Erwan Boroullec, SizeW313*D84*H370, Color꡼, ΡABS";
 
 		InternetAddress from = new InternetAddress("shop@example.com", "XMLMailBuilder饤󥷥å");
 		InternetAddress to = new InternetAddress(email, name);
 
 		String subject = "XMLMailBuilder饤󥷥å - ʸγǧ";
-		String text = name + "\n\n㤤夲꤬Ȥޤ";
+		String text = name
+				+ "\n\n㤤夲꤬Ȥޤ\n\nGIVE&TAKE (Beige)\n\nDesigned byRonan & Erwan Boroullec, SizeW313*D84*H370, Color꡼, ΡABS";
 
 		VelocityContext context = new VelocityContext();
 		context.put("customer", customer);
+		context.put("item", item);
 
 		// ᡼¹
 		Mail result = builder.buildMail(file, context);
@@ -238,6 +243,23 @@ public class JDomXMLMailBuilderTest extends TestCase {
 		assertEquals(text, result.getText());
 	}
 
+	public void testBuildMailFromMultipleMailsTemplate() throws Exception {
+		String path = "src/test/com/ozacc/mail/test-mail7-multiple.xml";
+		File file = new File(path);
+
+		Mail result1 = builder.buildMail(file, "first");
+		assertEquals("1", result1.getText());
+
+		Mail result2 = builder.buildMail(file, "second");
+		assertEquals("2", result2.getText());
+
+		try {
+			Mail result3 = builder.buildMail(file, "");
+		} catch (MailBuildException expected) {
+
+		}
+	}
+
 	public static class Customer {
 
 		private String name;
